@@ -16,7 +16,6 @@
     of CO2 into water.
 """
 # =================== Import Statements ==============================
-import unittest
 import pytest
 
 # Import specific pyomo objects
@@ -32,7 +31,6 @@ from pyomo.environ import (
 from pyomo.util.check_units import assert_units_consistent
 from pyomo.environ import units as pyunits
 
-import idaes.logger as idaeslog
 from idaes.core.util import scaling as iscale
 
 from idaes.core.util import get_solver
@@ -40,8 +38,6 @@ from idaes.core.util import get_solver
 # Import idaes methods to check the model during construction
 from idaes.core.util.model_statistics import (
     degrees_of_freedom,
-    fixed_variables_set,
-    activated_constraints_set,
 )
 
 # Import the idaes objects for Generic Properties and Reactions
@@ -57,8 +53,6 @@ from idaes.generic_models.properties.core.generic.generic_reaction import (
 from idaes.generic_models.properties.core.reactions.dh_rxn import constant_dh_rxn
 from idaes.generic_models.properties.core.reactions.equilibrium_constant import (
     gibbs_energy,
-)
-from idaes.generic_models.properties.core.reactions.equilibrium_constant import (
     van_t_hoff,
 )
 
@@ -88,6 +82,8 @@ from idaes.generic_models.properties.core.phase_equil.bubble_dew import IdealBub
 # Import log10 function from pyomo
 from pyomo.environ import log10
 
+import idaes.logger as idaeslog
+
 __author__ = "Srikanth Allu"
 
 # Create a thermo_config dictionary
@@ -184,10 +180,10 @@ thermo_config = {
                 "pressure_crit": (73.825e5, pyunits.Pa),
                 "temperature_crit": (304.23, pyunits.K),
                 "dens_mol_liq_comp_coeff": {
-                    "1": (2.768, pyunits.kmol * pyunits.m ** -3),
-                    "2": (0.26212, None),
-                    "3": (304.21, pyunits.K),
-                    "4": (0.2908, None),
+                    "1": (0.000789, pyunits.kmol * pyunits.m ** -3),
+                    "2": (0.000956, pyunits.dimensionless),
+                    "3": (500.78, pyunits.K),
+                    "4": (0.94599, pyunits.dimensionless),
                 },
                 "cp_mol_ig_comp_coeff": {
                     "A": (24.99735, pyunits.J / pyunits.mol / pyunits.K),
@@ -440,13 +436,13 @@ reaction_config = {
                 ("Liq", "OH_-"): 1,
             },
             "heat_of_reaction": constant_dh_rxn,
-            "equilibrium_constant": gibbs_energy,
+            "equilibrium_constant": van_t_hoff,
             "equilibrium_form": log_power_law_equil,
-            "concentration_form": ConcentrationForm.molarity,
+            "concentration_form": ConcentrationForm.moleFraction,
             "parameter_data": {
-                "dh_rxn_ref": (55.830, pyunits.kJ / pyunits.mol),
-                "ds_rxn_ref": (-80.7, pyunits.J / pyunits.mol / pyunits.K),
-                "T_eq_ref": (300, pyunits.K),
+                "dh_rxn_ref": (55.830, pyunits.J/pyunits.mol),
+                "k_eq_ref": (10**-14/55.2/55.2, pyunits.dimensionless),
+                "T_eq_ref": (298, pyunits.K),
                 # By default, reaction orders follow stoichiometry
                 #    manually set reaction order here to override
                 "reaction_order": {
@@ -467,11 +463,11 @@ reaction_config = {
             "heat_of_reaction": constant_dh_rxn,
             "equilibrium_constant": van_t_hoff,
             "equilibrium_form": log_power_law_equil,
-            "concentration_form": ConcentrationForm.molarity,
+            "concentration_form": ConcentrationForm.moleFraction,
             "parameter_data": {
                 "dh_rxn_ref": (0, pyunits.kJ / pyunits.mol),
-                "k_eq_ref": (1.7 * 10 ** -3, None),
-                "T_eq_ref": (300, pyunits.K),
+                "k_eq_ref": (1.7*10**-3, pyunits.dimensionless),
+                "T_eq_ref": (298, pyunits.K),
                 # By default, reaction orders follow stoichiometry
                 #    manually set reaction order here to override
                 "reaction_order": {
@@ -490,13 +486,13 @@ reaction_config = {
                 ("Liq", "HCO3_-"): 1,
             },
             "heat_of_reaction": constant_dh_rxn,
-            "equilibrium_constant": gibbs_energy,
+            "equilibrium_constant": van_t_hoff,
             "equilibrium_form": log_power_law_equil,
-            "concentration_form": ConcentrationForm.molarity,
+            "concentration_form": ConcentrationForm.moleFraction,
             "parameter_data": {
-                "dh_rxn_ref": (7.7, pyunits.kJ / pyunits.mol),
-                "ds_rxn_ref": (-95.8, pyunits.J / pyunits.mol / pyunits.K),
-                "T_eq_ref": (300, pyunits.K),
+                "dh_rxn_ref": (7.7, pyunits.kJ/pyunits.mol),
+                "k_eq_ref": (10**-6.35/55.2, pyunits.dimensionless),
+                "T_eq_ref": (298, pyunits.K),
                 # By default, reaction orders follow stoichiometry
                 #    manually set reaction order here to override
                 "reaction_order": {
@@ -515,13 +511,13 @@ reaction_config = {
                 ("Liq", "CO3_2-"): 1,
             },
             "heat_of_reaction": constant_dh_rxn,
-            "equilibrium_constant": gibbs_energy,
+            "equilibrium_constant": van_t_hoff,
             "equilibrium_form": log_power_law_equil,
-            "concentration_form": ConcentrationForm.molarity,
+            "concentration_form": ConcentrationForm.moleFraction,
             "parameter_data": {
-                "dh_rxn_ref": (14.9, pyunits.kJ / pyunits.mol),
-                "ds_rxn_ref": (-148.1, pyunits.J / pyunits.mol / pyunits.K),
-                "T_eq_ref": (300, pyunits.K),
+                "dh_rxn_ref": (14.9, pyunits.kJ/pyunits.mol),
+                "k_eq_ref": (10**-10.33/55.2, pyunits.dimensionless),
+                "T_eq_ref": (298, pyunits.K),
                 # By default, reaction orders follow stoichiometry
                 #    manually set reaction order here to override
                 "reaction_order": {
@@ -567,7 +563,7 @@ class TestCarbonationProcess:
 
         model.fs.unit.inlet.flow_mol.fix(10)
         model.fs.unit.inlet.pressure.fix(101325.0)
-        model.fs.unit.inlet.temperature.fix(300.0)
+        model.fs.unit.inlet.temperature.fix(298.0)
         model.fs.unit.inlet.mole_frac_comp[0, "H_+"].fix(0.0)
         model.fs.unit.inlet.mole_frac_comp[0, "OH_-"].fix(0.0)
         model.fs.unit.inlet.mole_frac_comp[0, "H2CO3"].fix(0.0)
@@ -610,6 +606,38 @@ class TestCarbonationProcess:
     @pytest.mark.component
     def test_scaling_equilibrium(self, equilibrium_config):
         model = equilibrium_config
+
+        # Equilibrium reactions have eps in the 'rxn_params'
+        factor = 1e-4
+        for rid in model.fs.rxn_params.equilibrium_reaction_idx:
+            scale = value(model.fs.unit.control_volume.reactions[0.0].k_eq[rid].expr)
+            # Want to set eps in some fashion similar to this
+            if scale < 1e-16:
+                model.fs.rxn_params.component("reaction_"+rid).eps.value = scale*factor
+            else:
+                model.fs.rxn_params.component("reaction_"+rid).eps.value = 1e-16*factor
+
+        for i in model.fs.unit.control_volume.equilibrium_reaction_extent_index:
+            scale = value(model.fs.unit.control_volume.reactions[0.0].k_eq[i[1]].expr)
+            iscale.set_scaling_factor(model.fs.unit.control_volume.equilibrium_reaction_extent[0.0,i[1]], 10/scale)
+            iscale.constraint_scaling_transform(model.fs.unit.control_volume.reactions[0.0].
+                    equilibrium_constraint[i[1]], 0.1)
+
+        # Next, try adding scaling for species
+        min = 1e-6
+        for i in model.fs.unit.control_volume.properties_out[0.0].mole_frac_phase_comp:
+            # i[0] = phase, i[1] = species
+            if model.fs.unit.inlet.mole_frac_comp[0, i[1]].value > min:
+                scale = model.fs.unit.inlet.mole_frac_comp[0, i[1]].value
+            else:
+                scale = min
+            iscale.set_scaling_factor(model.fs.unit.control_volume.properties_out[0.0].mole_frac_comp[i[1]], 10/scale)
+            iscale.set_scaling_factor(model.fs.unit.control_volume.properties_out[0.0].mole_frac_phase_comp[i], 10/scale)
+            iscale.set_scaling_factor(model.fs.unit.control_volume.properties_out[0.0].flow_mol_phase_comp[i], 10/scale)
+            iscale.constraint_scaling_transform(
+                model.fs.unit.control_volume.properties_out[0.0].component_flow_balances[i[1]], 10/scale)
+            iscale.constraint_scaling_transform(model.fs.unit.control_volume.material_balances[0.0,i[1]], 10/scale)
+
         iscale.calculate_scaling_factors(model.fs.unit)
 
         assert isinstance(model.fs.unit.control_volume.scaling_factor, Suffix)
@@ -632,7 +660,7 @@ class TestCarbonationProcess:
         model = equilibrium_config
         solver.options["bound_push"] = 1e-10
         solver.options["mu_init"] = 1e-6
-        model.fs.unit.initialize(optarg=solver.options)
+        model.fs.unit.initialize(optarg=solver.options, outlvl=idaeslog.DEBUG)
         assert degrees_of_freedom(model) == 0
 
     @pytest.mark.component
@@ -648,7 +676,7 @@ class TestCarbonationProcess:
     def test_solution_equilibrium(self, equilibrium_config):
         model = equilibrium_config
 
-        assert pytest.approx(300, rel=1e-5) == value(
+        assert pytest.approx(298, rel=1e-5) == value(
             model.fs.unit.outlet.temperature[0]
         )
         assert pytest.approx(10, rel=1e-5) == value(model.fs.unit.outlet.flow_mol[0])
@@ -662,19 +690,19 @@ class TestCarbonationProcess:
             )
             / 1000
         )
-        assert pytest.approx(55.1847856, rel=1e-5) == total_molar_density
+        assert pytest.approx(55.165246, rel=1e-5) == total_molar_density
         pH = -value(
             log10(model.fs.unit.outlet.mole_frac_comp[0, "H_+"] * total_molar_density)
         )
         pOH = -value(
             log10(model.fs.unit.outlet.mole_frac_comp[0, "OH_-"] * total_molar_density)
         )
-        assert pytest.approx(5.33698164, rel=1e-5) == pH
-        assert pytest.approx(8.598909, rel=1e-5) == pOH
+        assert pytest.approx(5.33989, rel=1e-4) == pH
+        assert pytest.approx(8.660655, rel=1e-4) == pOH
 
         CO2_sorbed = value(
             model.fs.unit.control_volume.properties_out[0.0].conc_mol_phase_comp[
                 ("Liq", "CO2")
             ]
         )
-        assert pytest.approx(27.5409958, rel=1e-5) == CO2_sorbed
+        assert pytest.approx(27.5312738, rel=1e-5) == CO2_sorbed
